local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local ChatModule = idTweaks:NewModule('Chat', 'AceConsole-3.0', 'AceEvent-3.0')

ChatModule.DEBUG = false

function ChatModule:Debug(...)
  if ChatModule.DEBUG then
    self:Print(...)
  end
end

function ChatModule:setDefaults()
  self:Debug('ChatModule:setDefaults')

  self.defaults = {
    profile = {
      slashCommands = {
        enabled = true
      },
      messageStyle = {
        enabled = true
      },
      experienceMessages = {
        enabled = true
      }
    }
  }
end

function ChatModule:setDatabase()
  self:Debug('ChatModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Chat', self.defaults)
end

function ChatModule:setOptions()
  self:Debug('ChatModule:setOptions')

  self.options = {
    name = 'Chat',
    desc = 'Options that change the chat frame and chat messages.',
    handler = self,
    type = 'group',
    args = {
      slashCommands = {
        order = 1,
        name = 'Slash commands',
        desc = 'Adds slash commands (/rl)',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.slashCommands.enabled
        end,
        set = function(_, value)
          self.db.profile.slashCommands.enabled = value
          self.slashCommands:checkEnabled()
        end
      },

      messageStyle = {
        order = 2,
        name = 'Style',
        desc = 'Change the style of your chat messages to a minimal format.',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.messageStyle.enabled
        end,
        set = function(_, value)
          self.db.profile.messageStyle.enabled = value
          self.messageStyle:checkEnabled()
        end
      },

      experienceMessages = {
        order = 3,
        name = 'Experience',
        desc = 'A custom experience message in your chat window, telling you how many repetitions you need to level up.',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.experienceMessages.enabled
        end,
        set = function(_, value)
          self.db.profile.experienceMessages.enabled = value
          self.experienceMessages:checkEnabled()
        end
      },
    }
  }
end

function ChatModule:registerOptions()
  self:Debug('ChatModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('Chat', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('Chat', 'Chat', 'idTweaks')
end

function ChatModule:OnInitialize()
  self:Debug('ChatModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function ChatModule:OnEnable()
  self:Debug('ChatModule:OnEnable')

  self:registerOptions()

  self.slashCommands:checkEnabled()
  self.messageStyle:checkEnabled()
  self.experienceMessages:checkEnabled()
end

function ChatModule:OnDisable()
  self:Debug('ChatModule:OnDisable')

  self.slashCommands:disable()
  self.messageStyle:disable()
  self.experienceMessages:disable()
end

ChatModule.slashCommands = {}

function ChatModule.slashCommands:checkEnabled()
  ChatModule:Debug('ChatModule.slashCommands:checkEnabled')

  if ChatModule.db.profile.slashCommands.enabled == true then
    self:enable()
  else
    self:disable()
  end
end

-- TODO: Add a slash command /tt that sends a tell to your target.
function ChatModule.slashCommands:enable()
  ChatModule:Debug('ChatModule.slashCommands:enable')

  -- TODO: Add a slash command /lfg to open up the group finder.
  SlashCmdList['IDTWEAKS_RELOAD'] = ReloadUI
  SLASH_IDTWEAKS_RELOAD1 = '/rl'
  SLASH_IDTWEAKS_RELOAD2 = '/reload'
end

function ChatModule.slashCommands:disable()
  ChatModule:Debug('ChatModule.slashCommands:disable')

  SlashCmdList['IDTWEAKS_RELOAD'] = nil
  SLASH_IDTWEAKS_RELOAD1 = nil
  SLASH_IDTWEAKS_RELOAD2 = nil
end

ChatModule.messageStyle = {
  filters = {
    CHAT_MSG_AFK = {
    },
    CHAT_MSG_COMBAT_FACTION_CHANGE = {
      ['Reputation with (.+) increased by (.+).'] = '+ %2 rep (%1)',
    },
    CHAT_MSG_COMBAT_HONOR_GAIN = {
    },
    CHAT_MSG_COMBAT_MISC_INFO = {
    },
    CHAT_MSG_CURRENCY = {
    },
    CHAT_MSG_DND = {
    },
    CHAT_MSG_GUILD_ACHIEVEMENT = {
      ['%%s has earned the achievement (.+)!'] = '\124cff00ff00g %1 %%s\124h',
    },
    CHAT_MSG_IGNORED = {
    },
    CHAT_MSG_LOOT = {
      ['You receive item: (.+)%.'] = '+ %1',
      ['You receive loot: (.+)%.'] = '+ %1',
      ['Received item: (.+)%.'] = '+ %1',
    },
    CHAT_MSG_MONEY = {
      ['Received (.+) Copper'] = '+ %1 c',
      ['Received (.+) Silver'] = '+ %1 s',
      ['Received (.+) Silver, (.+) Copper'] = '+ %1 s %2c',
      ['You loot (.+) Copper'] = '+ %1 c',
      ['You loot (.+) Silver'] = '+ %1 s',
    },
    CHAT_MSG_SKILL = {
      ['Your skill in (.+) has increased to (.+).'] = '+ %2 (%1)'
    },
    CHAT_MSG_SYSTEM = {
      ['Received (.+) Silver'] = '+ %1 s',
      ['Received (.+) of item: (.+).'] = '+ %2x%1',
      ['(.+) completed.'] = '- quest (%1)',
      ['Quest accepted: (.+)'] = '+ quest (%1)',
      ['Received item: (.+)%.'] = '+ %1',
      ['Experience gained: (.+).'] = '+ %1 xp',
      ['(.+) has come online.'] = '+ %1',
      ['(.+) has gone offline.'] = '- %1',
      ['You are now Busy: in combat'] = '+ Combat',
      ['You are no longer marked Busy.'] = '- Combat',
      ['Discovered (.+): (.+) experience gained'] = '+ %2 xp (%1)',
      ['Quest Accepted (.+)'] = '+ quest (%1)',
      ['You are now Away: AFK'] = '+ AFK',
      ['You are no longer Away.'] = '- AFK',
      ['You are no longer rested.'] = '- Rested',
      ['You don\'t meet the requirements for that quest.'] = '! quest (Requirements not met)',
    },
    CHAT_MSG_TARGETICONS = {
    },
    CHAT_MSG_TRADESKILLS = {
    },
  }
}

function ChatModule.messageStyle:checkEnabled()
  ChatModule:Debug('ChatModule.messageStyle:checkEnabled')

  if ChatModule.db.profile.messageStyle.enabled == true then
    self:enable()
  else
    self:disable()
  end
end

function ChatModule.messageStyle:enable()
  ChatModule:Debug('ChatModule.messageStyle:enable')

  for event, eventFilters in pairs(self.filters) do
    ChatFrame_AddMessageEventFilter(event, function(frame, ev, message, ...)
      for k,v in pairs(eventFilters) do
        if message:match(k) then
          message = message:gsub(k, v)
          return nil, message, ...
        end
      end
      return nil, message, ...
    end)
  end
end

function ChatModule.messageStyle:disable()
  ChatModule:Debug('ChatModule.messageStyle:disable')
end

ChatModule.experienceMessages = {
  xp = 0,
  xpMax = 0,
}

function ChatModule.experienceMessages:checkEnabled()
  ChatModule:Debug('ChatModule.experienceMessages:checkEnabled', ChatModule.db.profile.experienceMessages.enabled)

  if ChatModule.db.profile.experienceMessages.enabled == true then
    self:enable()
  else
    self:disable()
  end
end

function ChatModule.experienceMessages:enable()
  ChatModule:Debug('ChatModule.experienceMessages:enable')

  self.xp = UnitXP('player')
  self.xpMax = UnitXPMax('player')

  ChatModule:RegisterEvent('CHAT_MSG_COMBAT_XP_GAIN')
  ChatModule:RegisterEvent('PLAYER_XP_UPDATE')
end

function ChatModule.experienceMessages:disable()
  ChatModule:Debug('ChatModule.experienceMessages:disable')

  ChatModule:UnregisterEvent('CHAT_MSG_COMBAT_XP_GAIN')
  ChatModule:UnregisterEvent('PLAYER_XP_UPDATE')
end

function ChatModule.experienceMessages:printExperienceMessage()
  ChatModule:Debug('ChatModule.experienceMessages:printExperienceMessage')

  -- Get the old experience values.
  local oldXp = ChatModule.experienceMessages.xp
  local oldXpMax = ChatModule.experienceMessages.xpMax
  --self:Print('oldXp', oldXp)
  --self:Print('oldXpMax', oldXpMax)

  -- Get the new experience values.
  local newXp = UnitXP('player')
  local newXpMax = UnitXPMax('player')
  --self:Print('newXp', newXp)
  --self:Print('newXpMax', newXpMax)

  -- Calculate the difference.
  local gainedXp = newXp - oldXp
  local gainedXpMax = newXpMax - oldXpMax
  --self:Print('gainedXp', gainedXp)
  --self:Print('gainedXpMax', gainedXpMax)

  local gainedXpPercentage = gainedXp / newXpMax * 100
  local gainedXpMaxPercentage = gainedXpMax / oldXpMax * 100
  --self:Print('gainedXpPercentage', gainedXpPercentage)
  --self:Print('gainedXpMaxPercentage', gainedXpMaxPercentage)

  local gainedXpRepetitionsLeft = (newXpMax - newXp) / gainedXp
  --self:Print('gainedXpRepetitionsLeft', gainedXpRepetitionsLeft)

  if gainedXpMax > 0 then
    print(string.format('+ XP MAX: %s (%.f%%)', gainedXpMax, gainedXpMaxPercentage))
  end

  if gainedXp > 0 then
    print(string.format('+ XP: %s (%.2f%%), left: %s, reps: %.1f', gainedXp, gainedXpPercentage, newXpMax - newXp, gainedXpRepetitionsLeft))
  end

  -- Save the new experience values.
  self.xp = newXp
  self.xpMax = newXpMax
end

function ChatModule:CHAT_MSG_COMBAT_XP_GAIN()
  self:Debug('ChatModule:CHAT_MSG_COMBAT_XP_GAIN')

  if not ChatModule.db.profile.experienceMessages.enabled then return end

  self.experienceMessages:printExperienceMessage()
end

function ChatModule:PLAYER_XP_UPDATE()
  self:Debug('ChatModule:PLAYER_XP_UPDATE')

  if not ChatModule.db.profile.experienceMessages.enabled then return end

  self.experienceMessages:printExperienceMessage()
end

idTweaks.ChatModule = ChatModule
